@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
->>> s = SubRipFile("""1
+>>> s = SubRipFile.from_string("""1
 ... 00:00:02,207 --> 00:00:03,221
 ... ÉCHEC !
 ...
@@ -21,12 +21,50 @@
 ... - 1 800 milliards de milliards.
 ... - Splendide !
 ...
-... 6
+... 7
 ... 00:00:43,410 --> 00:00:45,136
 ... Sans compter les revenus subsidiaires
 ... """)
 >>> len(s)
 6
+>>> s.clean_ids()
+>>> s[5].id
+6
+>>> s[-1].end.seconds
+45
+>>> part = SubRipFile(i for i in s if i.end > (0, 0, 40))
+>>> len(part)
+2
+>>> part = SubRipFile(i for i in s if i.end > {'seconds': 40})
+>>> len(part)
+2
+>>> part.shift(seconds=-5)
+>>> part[1].start.minutes += 5
+>>> item = s[-1]
+>>> item.end.seconds
+40
+>>> tuple(item.start), tuple(SubRipTime(*tuple(item.start)))
+((0, 5, 38, 410), (0, 5, 38, 410))
+>>> tuple(item.start) == SubRipTime(*tuple(item.start))
+True
+>>> item.start <= (0, 5, 39)
+True
+>>> item.start >= (0, 5, 37)
+True
+>>> item.start == (0, 5, 38, 410)
+True
+
+>>> t= SubRipTime(micro_seconds=1)
+>>> t.seconds, t.ordinal
+(0, 1)
+>>> t.seconds += 1
+>>> t.seconds, t.ordinal
+(1, 1001)
+>>> tuple(t + {'seconds': 1})
+(0, 0, 2, 1)
+>>> tuple(t - {'seconds': 1})
+(0, 0, 0, 1)
+
 '''
 __all__ = ['SubRipFile', 'SubRipItem', 'SubRipTime']
 import re
@@ -95,13 +133,43 @@ class SubRipTime(object):
         return unicode(self.STRING_FORMAT) % (self.hours,
             self.minutes, self.seconds, self.micro_seconds)
 
+    def __cmp__(self, other):
+        return cmp(self.ordinal, self._coerce(other).ordinal)
+
+    def __add__(self, other):
+        return self.from_ordinal(self.ordinal + self._coerce(other).ordinal)
+
+    def __iadd__(self, other):
+        self.ordinal += self._coerce(other).ordinal
+
+    def __sub__(self, other):
+        return self.from_ordinal(self.ordinal - self._coerce(other).ordinal)
+
+    def __isub__(self, other):
+        self.ordinal -= self._coerce(other).ordinal
+
+    @classmethod
+    def _coerce(cls, other):
+        if isinstance(other, SubRipTime):
+            return other
+        try:
+            return cls(**other)
+        except TypeError:
+            return cls(*other)
+
+    def __iter__(self):
+        yield self.hours
+        yield self.minutes
+        yield self.seconds
+        yield self.micro_seconds
+
     def shift(self, *args, **kwargs):
         """
         shift(hours, minutes, seconds, micro_seconds)
 
         All arguments are optional and have a default value of 0.
         """
-        self.ordinal += self.__class__(*args, **kwargs).ordinal
+        self += self.__class__(*args, **kwargs)
 
     @classmethod
     def from_ordinal(cls, ordinal):
@@ -134,6 +202,10 @@ class SubRipItem(object):
         return u'%s\n%s --> %s\n%s\n\n' % (self.id,
             self.start, self.end, self.sub_title)
 
+    def __cmp__(self, other):
+        return cmp(self.start, other.start) \
+            or cmp(self.end, other.end)
+
     def shift(self, *args, **kwargs):
         """
         shift(hours, minutes, seconds, micro_seconds)
@@ -162,32 +234,41 @@ class SubRipFile(object, UserList):
     Provide a pure Python mapping on all metadatas.
     """
 
-    def __init__(self, path='', encoding='utf-8'):
+    def __init__(self, items=None):
+        UserList.__init__(self, items or [])
+
+    @classmethod
+    def open(cls, path='', encoding='utf-8'):
         """
-        SubRipFile([path, [encoding]])
+        open([path, [encoding]])
 
         Encoding is set to utf-8 as default.
         """
-        UserList.__init__(self)
-        self.encoding = encoding
-        self.path = path
+        new_file = cls()
+        new_file.encoding = encoding
+        new_file.path = path
 
-        if path:
-            if exists(path) and isfile(path):
-                source_file = open(path, 'rU')
+        if isinstance(path, basestring):
+            source_file = open(path, 'rU')
+        else:
+            source_file = path
+
+        string_buffer = StringIO()
+        for line in chain(source_file, '\n'):
+            if line.strip():
+                string_buffer.write(line)
             else:
-                source_file = StringIO(path)
-            string_buffer = StringIO()
-            for line in chain(source_file, '\n'):
-                if line.strip():
-                    string_buffer.write(line)
-                else:
-                    string_buffer.seek(0)
-                    source = unicode(string_buffer.read(), self.encoding)
-                    new_item = SubRipItem.from_string(source)
-                    self.data.append(new_item)
-                    string_buffer.truncate(0)
-            source_file.close()
+                string_buffer.seek(0)
+                source = unicode(string_buffer.read(), new_file.encoding)
+                new_item = SubRipItem.from_string(source)
+                new_file.append(new_item)
+                string_buffer.truncate(0)
+        source_file.close()
+        return new_file
+
+    @classmethod
+    def from_string(cls, source):
+        return cls.open(StringIO(source))
 
     def shift(self, *args, **kwargs):
         """
@@ -200,6 +281,11 @@ class SubRipFile(object, UserList):
         for item in self:
             item.shift(*args, **kwargs)
 
+    def clean_ids(self):
+        self.sort()
+        for index, item in enumerate(self):
+            self[index].id = index + 1
+
     def save(self, path=None, encoding=None):
         """
         save([save_path[, encoding]])
